/*------------------------------------------------------------------------------
--                                                                            --
--       This software is confidential and proprietary and may be used        --
--        only as expressly authorized by a licensing agreement from          --
--                                                                            --
--                            Hantro Products Oy.                             --
--                                                                            --
--      In the event of publication, the following notice is applicable:      --
--                                                                            --
--                   (C) COPYRIGHT 2003 HANTRO PRODUCTS OY                    --
--                            ALL RIGHTS RESERVED                             --
--                                                                            --
--          The entire notice above must be reproduced on all copies.         --
--                                                                            --
--------------------------------------------------------------------------------
--
--  Abstract : Encoder HW register access functions
--
--------------------------------------------------------------------------------
--
--  Version control information, please leave untouched.
--
--  $RCSfile: encregdrv2.c,v $
--  $Date: 2005/12/20 16:21:36 $
--  $Revision: 1.16 $
--
------------------------------------------------------------------------------*/
#include "ewl.h"
#include "basetype.h"
#include "encregdrv.h"
#include "EncSim.h"

/* 1. (AMBAIFE) Encoder Register Offsets   */

#define ENCREG(nr)  (nr<<2)

/* 2. Mask fields */
#define mask_1b         (u32)0x00000001
#define mask_2b         (u32)0x00000003
#define mask_3b         (u32)0x00000007
#define mask_4b         (u32)0x0000000F
#define mask_5b         (u32)0x0000001F
#define mask_6b         (u32)0x0000003F
#define mask_8b         (u32)0x000000FF
#define mask_16b        (u32)0x0000FFFF

/* 3. Encoder register access drivers */

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x0C) ************************/
/******************************************************************************/

/*******************************************************************************
 Function name   : SetEncStrmBuf3Lim
 Description     : Set the initial limit in the DCT stream buffer when the
                   VOP should be discarded.
 Return type     : void 
 Argument        : u32 offset
*******************************************************************************/
void SetEncStrmBuf3Lim(u32 offset)
{
    u32 val;

    ASSERT(offset < (1 << 16));

    val = EWL_ReadReg(0x0C);
    val = (val & ~(mask_16b << 16)) | (offset << 16);
    EWL_WriteReg(0x0C, val);
}

/*******************************************************************************
 Function name   : GetEncStrmBuf3Lim
 Description     : Get the current limit in the DCT stream buffer
 Return type     : u32 
*******************************************************************************/
u32 GetEncStrmBuf3Lim()
{
    u32 val;

    val = EWL_ReadReg(0x0C);
    val = (val >> 16);
    return val;
}

/*******************************************************************************
 Function name   : SetEncStrmBuf1Lim
 Description     : Set the limit in the final stream buffer
 Return type     : void 
 Argument        : u32 offset
*******************************************************************************/
void SetEncStrmBuf1Lim(u32 offset)
{
    u32 val;

    ASSERT(offset < (1 << 16));

    val = EWL_ReadReg(0x0C);
    val = (val & ~mask_16b) | (offset);
    EWL_WriteReg(0x0C, val);
}

/*******************************************************************************
 Function name   : GetEncStrmBuf1Lim
 Description     : Get the limit in the final stream buffer
 Return type     : u32 
*******************************************************************************/
u32 GetEncStrmBuf1Lim()
{
    u32 val;

    val = EWL_ReadReg(0x0C);
    val = (val & mask_16b);
    return val;
}

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x10) ************************/
/******************************************************************************/

/*******************************************************************************
 Function name   : SetEncIrqClr
 Description     : Clear the IRQ line
 Return type     : void 
*******************************************************************************/
void SetEncIrqClr()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = val & (~mask_1b);
    EWL_WriteReg(0x10, val);
}

/*******************************************************************************
 Function name   : SetEncStrmStatClr
 Description     : Clear the stream status
 Return type     : void 
*******************************************************************************/
void SetEncStrmStatClr()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = val & ~(mask_1b << 2);
    EWL_WriteReg(0x10, val);
}

/*******************************************************************************
 Function name   : GetEncStrmStat
 Description     : Get the stream status
 Return type     : u32
                   0    No stream was generated after previous interrupt
                   1    Stream was generated after previous interrupt
*******************************************************************************/
u32 GetEncStrmStat()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = (val >> 2) & mask_1b;
    return val;
}

/*******************************************************************************
 Function name   : SetEncEnable
 Description     : Enable/Disable ASIC
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncEnable(u32 enable)
{
    u32 val;

    if(enable)
    {
        /* enable ASIC clock */
        val = EWL_ReadReg(0x0);
        val = val | (mask_1b << 12);
        EWL_WriteReg(0x0, val);
        
        /* enable ASIC */
        val = EWL_ReadReg(0x10);
        val = val | (mask_1b << 1);
        EWL_WriteReg(0x10, val);
    }
    else
    {
        /* disable ASIC first */
        val = EWL_ReadReg(0x10);
        val = val & ~(mask_1b << 1);
        EWL_WriteReg(0x10, val);

        SetEncIrqClr(); /* clear any IRQ */
        SetEncIrqStatClr(IRQ_ALL);  /* clear pause status */
    }

}

/*******************************************************************************
 Function name   : GetEncEnaStat
 Description     : Get ASIC's enable status
 Return type     : u32 
*******************************************************************************/
u32 GetEncEnaStat()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = (val >> 1) & mask_1b;
    return val;
}

/*******************************************************************************
 Function name : SetEncHWIrqEnable
 Description   : Enabel/Disable the HW interrupt generation
 Return type   : void 
 Argument      : u32 error_irq - non zero to enable the error IRQ
 Argument      : u32 enc_irq - non zero to enable the encoder IRQ
*******************************************************************************/
void SetEncHWIrqEnable(u32 error_irq, u32 enc_irq)
{
    u32 val;

    val = EWL_ReadReg(0x10);

    if(error_irq != 0)
        val = val | (mask_1b << 8);
    else
        val = val & ~(mask_1b << 8);

    if(enc_irq != 0)
        val = val | (mask_1b << 7);
    else
        val = val & ~(mask_1b << 7);

    EWL_WriteReg(0x10, val);
}

/*******************************************************************************
 Function name   : GetEncHWIrqEnable
 Description     : Get the IRQ enable status
 Return type     : u32 
*******************************************************************************/
u32 GetEncHWIrqEnable()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = (mask_2b & (val >> 7));
    return val;
}

/*******************************************************************************
 Function name   : SetEncIrqStatClr
 Description     : Clear the IRQ status bits
 Return type     : void 
*******************************************************************************/
void SetEncIrqStatClr(u32 mask)
{
    u32 val;
    
    if ((mask & IRQ_VOP_RDY) == IRQ_VOP_RDY)
    {
        /* disable ASIC clock when the VOP is ready */
        val = EWL_ReadReg(0x0);
        val = val & ~(mask_1b << 12);
        EWL_WriteReg(0x0, val);
    }

    val = EWL_ReadReg(0x10);
    val = val & (~((mask & mask_4b) << 3));
    EWL_WriteReg(0x10, val);
}

/*******************************************************************************
 Function name   : GetEncIrqStat
 Description     : Get the IRQ status
 Return type     : u32 
*******************************************************************************/
u32 GetEncIrqStat()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = (mask_4b & (val >> 3));
    return val;
}

/*******************************************************************************
 Function name   : SetEncMbsInCol
 Description     : Set the picture height in macroblocks
 Return type     : void 
 Argument        : u32 mbs
*******************************************************************************/
void SetEncMbsInCol(u32 mbs)
{
    u32 val;

    ASSERT(mbs >= 6 && mbs <= 30);

    val = EWL_ReadReg(0x10);
    val = (val & ~(mask_5b << 10)) | (mbs << 10);
    EWL_WriteReg(0x10, val);
}

/*******************************************************************************
 Function name   : GetEncMbsInCol
 Description     : Get the picture height in macroblocks
 Return type     : u32 
*******************************************************************************/
u32 GetEncMbsInCol()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = (mask_5b & (val >> 10));
    return val;
}

/*******************************************************************************
 Function name   : SetEncMbsInRow
 Description     : Set the picture width in macroblocks
 Return type     : void 
 Argument        : u32 mbs
*******************************************************************************/
void SetEncMbsInRow(u32 mbs)
{
    u32 val;

    ASSERT(mbs >= 6 && mbs <= 40);

    val = EWL_ReadReg(0x10);
    val = (val & ~(mask_6b << 15)) | (mbs << 15);
    EWL_WriteReg(0x10, val);
}

/*******************************************************************************
 Function name   : GetEncMbsInRow
 Description     : Get the picture width in macroblock
 Return type     : u32 
*******************************************************************************/
u32 GetEncMbsInRow()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = (val >> 15) & mask_6b;
    return val;
}

/*******************************************************************************
 Function name   : GetEncVpNr
 Description     : Get the number of VPs created
 Return type     : u32 
*******************************************************************************/
u32 GetEncVpNr()
{
    u32 val;

    val = EWL_ReadReg(0x10);
    val = (val >> 22);
    return val;
}

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x14) ************************/
/******************************************************************************/

/*******************************************************************************
 Function name   : SetEncVpSizeBase
 Description     : Set the base address of a buffer where the VPs' sizes are
                   written
 Return type     : void 
 Argument        : u32 bus_address
*******************************************************************************/
void SetEncVpSizeBase(u32 bus_address)
{
    ASSERT((bus_address & 0x03) == 0);  /* word aligned address */
    EWL_WriteReg(0x14, bus_address);
}

/*******************************************************************************
 Function name   : GetEncVpSizeBase
 Description     : Get the base address of the buffer where the VPs' sizes are
                   written
 Return type     : u32 
*******************************************************************************/
u32 GetEncVpSizeBase()
{
    u32 val;

    val = EWL_ReadReg(0x14);

    return val;
}

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x18) ************************/
/******************************************************************************/

/*******************************************************************************
 Function name   : SetEncNewLumaBase
 Description     : Set the base address for the luminance component of the
                   image to be encoded
 Return type     : void 
 Argument        : u32 bus_address
*******************************************************************************/
void SetEncNewLumaBase(u32 bus_address)
{
    /* Bits 1:0 are used for camera stabilization of luminance */
    EWL_WriteReg(0x18, bus_address);
}

/*******************************************************************************
 Function name   : GetEncNewLumaBase
 Description     : Get the base address of the new input image's luminance data
 Return type     : u32 
*******************************************************************************/
u32 GetEncNewLumaBase()
{
    u32 val;

    val = EWL_ReadReg(0x18);

    return val;
}

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x1C) ************************/
/******************************************************************************/
void SetEncNewChromaUBase(u32 bus_address)
{
    /* Bits 1:0 are used for camera stabilization of chrominance */
    EWL_WriteReg(0x1C, bus_address);
}

u32 GetEncNewChromaUBase()
{
    u32 val;

    val = EWL_ReadReg(0x1C);

    return val;
}

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x20) ************************/
/******************************************************************************/
void SetEncNewChromaVBase(u32 bus_address)
{
    ASSERT((bus_address & 0x03) == 0);  /* word aligned address */
    EWL_WriteReg(0x20, bus_address);
}

u32 GetEncNewChromaVBase()
{
    u32 val;

    val = EWL_ReadReg(0x20);

    return val;
}

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x24) ************************/
/******************************************************************************/

/*******************************************************************************
 Function name   : SetEncVopTimeIncr
 Description     : Set the vop time increment value
 Return type     : void 
 Argument        : u32 vop_time_incr
                     VopTimeIncrBits specifies the amount of bits starting from
                     MSB that are written to the stream.
                     So if VopTimeIncrBits == 5
                     and vop_time_incr == '0001 0000 0000 0000'
                     bits '00010' are written to the stream
*******************************************************************************/
void SetEncVopTimeIncr(u32 vop_time_incr)
{
    u32 val;

    ASSERT(vop_time_incr < ((1 << 16) - 1));

    val = EWL_ReadReg(0x24);
    val = (val & ~(mask_16b << 16)) | (vop_time_incr << 16);
    EWL_WriteReg(0x24, val);
}

/*******************************************************************************
 Function name   : GetEncMbsInRow
 Description     : Set the vop time increment value
 Return type     : u32 
*******************************************************************************/
u32 GetEncVopTimeIncr()
{
    u32 val;

    val = EWL_ReadReg(0x24);
    val = (val >> 16);
    return val;
}

/*******************************************************************************
 Function name   : SetEncVopTimeIncrBits
 Description     : Set how many bits are used for the vop time increment value
 Return type     : void 
 Argument        : u32 bits [0 - 15]
                   0    16 valid bits
                   1    1  valid bit
                   2    2  valid bits
                   .    .
                   15   15 valid bits
*******************************************************************************/
void SetEncVopTimeIncrBits(u32 bits)
{
    u32 val;

    ASSERT(bits < 16);

    val = EWL_ReadReg(0x24);
    val = (val & ~(mask_4b << 12)) | (bits << 12);
    EWL_WriteReg(0x24, val);
}

/*******************************************************************************
 Function name   : GetEncMbsInRow
 Description     : Get how many bits are used for the vop time increment value
 Return type     : u32 
*******************************************************************************/
u32 GetEncVopTimeIncrBits()
{
    u32 val;

    val = EWL_ReadReg(0x24);
    val = (val >> 12) & mask_4b;
    return val;
}

/*******************************************************************************
 Function name   : SetEncVopHdrLenght
 Description     : Set how many bits are used for the vop header
 Return type     : void 
 Argument        : u32 bits
*******************************************************************************/
void SetEncVopHdrLenght(u32 bits)
{
    u32 val;

    ASSERT(bits < 256);

    val = EWL_ReadReg(0x24);
    val = (val & ~(mask_8b << 4)) | (bits << 4);
    EWL_WriteReg(0x24, val);
}

/*******************************************************************************
 Function name   : GetEncVopHdrLenght
 Description     : Get how many bits are used for the vop header
 Return type     : u32 
*******************************************************************************/
u32 GetEncVopHdrLenght()
{
    u32 val;

    val = EWL_ReadReg(0x24);
    val = (val >> 4) & mask_8b;
    return val;
}

/*******************************************************************************
 Function name   : SetEncHecEnable
 Description     : Enable/disable HEC
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncHecEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x24);
    val = (val & ~(mask_1b << 3)) | (enable << 3);
    EWL_WriteReg(0x24, val);
}

/*******************************************************************************
 Function name   : GetEncHecEnable
 Description     : Get HEC status
 Return type     : u32 
*******************************************************************************/
u32 GetEncHecEnable()
{
    u32 val;

    val = EWL_ReadReg(0x24);
    val = (val >> 3) & mask_1b;
    return val;
}
