/*------------------------------------------------------------------------------
--                                                                            --
--       This software is confidential and proprietary and may be used        --
--        only as expressly authorized by a licensing agreement from          --
--                                                                            --
--                            Hantro Products Oy.                             --
--                                                                            --
--      In the event of publication, the following notice is applicable:      --
--                                                                            --
--                   (C) COPYRIGHT 2003 HANTRO PRODUCTS OY                    --
--                            ALL RIGHTS RESERVED                             --
--                                                                            --
--          The entire notice above must be reproduced on all copies.         --
--                                                                            --
--------------------------------------------------------------------------------
--
--  Project  : 1200s
--
--  Abstract : Encoder wrapper module
--
--------------------------------------------------------------------------------
--
--  Version control information, please leave untouched.
--
--  $RCSfile: ewl.c,v $
--  $Author: seto $
--  $Date: 2005/11/03 17:34:36 $
--  $Revision: 1.7 $
--  $Name: Rel5251-HW_Encoder_Control_Code_GLA-Linux_2_6-iMX31-src-1_1 $
--
------------------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include <string.h>

#include <pthread.h>

#include <errno.h>

#include <assert.h>

#include "basetype.h"
#include "mxc_hmp4e.h"

#include "encregdrv.h"

#include "ewl.h"

/* the encoder device driver nod */
const char *dev = "/dev/hmp4e";

/* wrapper information */
struct enc_t
{
    int fd; /* encoder device file */
    u32 *pMemBase;  /* Virtual base */
    u32 BusBase;    /* Bus base */
    u32 MemSize;    /* Size of memory */
    u32 *pFreeMem;  /* Next free address */
    sigset_t sigset;
};


struct enc_t *enc = NULL;


/* a mutex protecting the wrapper init */
pthread_mutex_t init_mutex = PTHREAD_MUTEX_INITIALIZER;

/*******************************************************************************
 Function name   : EWL_Init
 Description     : Allocate resources and setup the wrapper module
 Return type     : ewl_ret 
 Argument        : void
*******************************************************************************/
i32 EWL_Init(void)
{

  /*  unsigned int hw_offset;*/
    unsigned int oflags;

    pthread_mutex_lock(&init_mutex);

    if(enc == NULL)
    {
        if((enc = (struct enc_t *) malloc(sizeof(struct enc_t))) == NULL)
        {
            PTRACE("Init: failed to alloc enc_t struct\n");
            pthread_mutex_unlock(&init_mutex);
            return EWL_NOK;
        }
        enc->fd = -1;
    }
    else
    {
        pthread_mutex_unlock(&init_mutex);
        return EWL_NOK; /* just one instance */
    }

    pthread_mutex_unlock(&init_mutex);

    /* open the device */
    enc->fd = open(dev, O_RDWR | O_SYNC);
    if(enc->fd == -1)
    {
        PTRACE("Init: failed to open '%s'\n", dev);
        goto err;
    }

    /* get buffer size */
    ioctl(enc->fd, HMP4E_IOCGBUFSIZE, &enc->MemSize);

    /* get the buffer's bus address */
    ioctl(enc->fd, HMP4E_IOCGBUFBUSADDRESS, &enc->BusBase);

    /* map the buffer to user space */
    enc->pMemBase =
        (u32 *) mmap(0, enc->MemSize, PROT_READ | PROT_WRITE, MAP_SHARED,
                     enc->fd, 0);
    if(enc->pMemBase == MAP_FAILED)
    {
        PTRACE("Init: failed to mmap mem buffers\n");
        goto err;
    }

    PTRACE("Init: membuf size %d bytes, virt %08x, bus %08x\n", enc->MemSize,
           (u32) enc->pMemBase, enc->BusBase);

    memset(enc->pMemBase, 0, enc->MemSize);

    enc->pFreeMem = enc->pMemBase;

    /* asynchronus notification */
    fcntl(enc->fd, F_SETOWN, getpid()); /* this process will receive SIGIO */
    oflags = fcntl(enc->fd, F_GETFL);
    fcntl(enc->fd, F_SETFL, oflags | FASYNC);   /* set ASYNC notification flag */

    sigemptyset(&enc->sigset);
    sigaddset(&enc->sigset, SIGIO); /* HW IRQ */
    pthread_sigmask(SIG_BLOCK, &enc->sigset, NULL);

    return EWL_OK;

  err:
    EWL_Release();
    return EWL_NOK;
}

/*******************************************************************************
 Function name   : EWL_Release
 Description     : Release the wrapper module by freeing all the resources
 Return type     : ewl_ret 
 Argument        : void
*******************************************************************************/
i32 EWL_Release(void)
{
    assert(enc != NULL);

    pthread_mutex_lock(&init_mutex);

    if(enc->pMemBase != MAP_FAILED)
        munmap((void *) enc->pMemBase, enc->MemSize);

    if(enc->fd != -1)
        close(enc->fd);

    free(enc);
    enc = NULL;

    pthread_mutex_unlock(&init_mutex);

    return (EWL_OK);
}

/*******************************************************************************
 Function name   : EnableRegAccess
 Description     : Enable write access to given register if necessary
 Return type     : u32
 Argument        : u32
*******************************************************************************/
static inline u32 EnableRegAccess( u32 offset )
{
    #define IPS  0
    #define HCLK 1
    write_t bwrite;
    static const u32 clockSource[] =
    {
        /*0x00*/ IPS,   /*0x04*/ IPS,   /*0x08*/ IPS,   /*0x0C*/ IPS,
        /*0x10*/ HCLK,  /*0x14*/ IPS,   /*0x18*/ IPS,   /*0x1C*/ IPS,
        /*0x20*/ IPS,   /*0x24*/ IPS,   /*0x28*/ IPS,   /*0x2C*/ HCLK,
        /*0x30*/ IPS,   /*0x34*/ IPS,   /*0x38*/ IPS,   /*0x3C*/ IPS,
        /*0x40*/ IPS,   /*0x44*/ IPS,   /*0x48*/ IPS,   /*0x4C*/ IPS,
        /*0x50*/ IPS,   /*0x54*/ IPS,   /*0x58*/ IPS,   /*0x5C*/ HCLK,
        /*0x60*/ HCLK,  /*0x64*/ HCLK,  /*0x68*/ HCLK,  /*0x6C*/ HCLK,
        /*0x70*/ IPS,   /*0x74*/ IPS,   /*0x78*/ IPS,   /*0x7C*/ IPS,
        /*0x80*/ HCLK,  /*0x84*/ IPS,   /*0x88*/ IPS
    };

	

    static const u32 clockSourceSize = sizeof(clockSource)/sizeof(clockSource[0]);
    u32 clearmask = 0;
    offset /= 4;
    
    if( offset < clockSourceSize && clockSource[offset] == HCLK  )
    {   
	u32 val = 0x00;
	ioctl(enc->fd, HMP4E_IOCXREGREAD, &val);
	clearmask = ~val & (1<<12);
	bwrite.data = val | 1<<12;
	bwrite.offset = 0x00;
	ioctl(enc->fd, HMP4E_IOCSREGWRITE, &bwrite);
    }

    return clearmask;
}

/*******************************************************************************
 Function name   : RestoreRegAccess
 Description     : Restore settings for register write access
 Return type     : void
 Argument        : u32
*******************************************************************************/
static inline void RestoreRegAccess( u32 clearmask )
{
    write_t bwrite;
    if( clearmask )
    {	
	u32 val = 0x00;
	ioctl(enc->fd, HMP4E_IOCXREGREAD, &val);
    	val &= ~clearmask;
	bwrite.data = val;	
	bwrite.offset = 0x00;
	ioctl(enc->fd, HMP4E_IOCSREGWRITE, &bwrite);
    }
}

/*******************************************************************************
 Function name   : EWL_WriteReg
 Description     : Set the content of a hadware register
 Return type     : void 
 Argument        : u32 offset
 Argument        : u32 val
*******************************************************************************/
void EWL_WriteReg(u32 offset, u32 val)
{
    u32 saved;
    write_t bwrite;

    saved = EnableRegAccess( offset );

    PTRACE("Write reg at offset 0x%02X with value %08X\n", offset, val);
    bwrite.data = val;
    bwrite.offset = offset;
    ioctl(enc->fd, HMP4E_IOCSREGWRITE, &bwrite);

    RestoreRegAccess( saved );
}

/*******************************************************************************
 Function name   : EWL_ReadReg
 Description     : Retrive the content of a hadware register
 Return type     : u32 
 Argument        : u32 offset
*******************************************************************************/
u32 EWL_ReadReg(u32 offset)
{
    u32 val, saved;
 
    saved = EnableRegAccess( offset );
    
    val = offset;
    ioctl(enc->fd, HMP4E_IOCXREGREAD, &val);
    PTRACE("Read reg at offset 0x%02X --> %08X\n", offset * 4, val);
    
    RestoreRegAccess( saved );
    
    return val;
}

/*******************************************************************************
 Function name   : EWL_MallocLinear
 Description     : Allocate a linear buffer in the physical memory, accessible
                   by the hardware
 Return type     : u32 * 
 Argument        : u32 size
*******************************************************************************/

u32 *EWL_MallocLinear(u32 size)
{
    u32 *pNextFree, *pAlloc;

    assert(enc != NULL && size > 0 && size <= enc->MemSize);

    /* Check size not zero */
    if(size == 0)
    {
        return (u32 *) NULL;
    }

    size = (size + 3) & (~0x03);    /* round up to a 4 multiple */

    pNextFree = enc->pFreeMem + (size / 4);

    if(pNextFree >= enc->pMemBase + (enc->MemSize / 4))
        return (u32 *) NULL;

    pAlloc = enc->pFreeMem;
    enc->pFreeMem = pNextFree;

    PTRACE("MallocLinear: %d bytes --> %08x\n", size, (u32) pAlloc);

    return (u32 *) pAlloc;
}

/*******************************************************************************
 Function name   : EWL_FreeLinear
 Description     : Free a memory buffer allocated with EWL_MallocLinear
 Return type     : void 
 Argument        : u32 * pVirt
*******************************************************************************/
void EWL_FreeLinear(u32 * pVirt)
{
    /* nothing to do, everything is released in the device driver */
    PTRACE("FreeLinear: %08x\n", (u32) pVirt);

    assert(enc != NULL);

    if(pVirt == NULL)
        return;

    assert(pVirt >= enc->pMemBase &&
           pVirt < (enc->pMemBase + (enc->MemSize / 4)));
}

/*******************************************************************************
 Function name   : EWL_Virt2Bus
 Description     : Get the bus address from a virtual address
 Return type     : u32 
 Argument        : u32 * pVirt
*******************************************************************************/
u32 EWL_Virt2Bus(u32 * pVirt)
{
    u32 offset;

    assert(enc != NULL);
    assert(pVirt >= enc->pMemBase &&
           pVirt < (enc->pMemBase + (enc->MemSize / 4)));

    offset = (u32) pVirt - (u32) enc->pMemBase;

    PTRACE("Virt2Bus: %08x --> %08x\n", (u32) pVirt, enc->BusBase + offset);
    return enc->BusBase + offset;
}

/*******************************************************************************
 Function name   : EWL_PostHwRdy
 Description     : Signal the encoder semaphore
 Return type     : ewl_ret 
 Argument        : void
*******************************************************************************/
i32 EWL_PostHwRdy(void)
{
    return (EWL_OK);
}

/*******************************************************************************
 Function name   : EWL_WaitHwRdy
 Description     : Wait for the encoder semaphore
 Return type     : ewl_ret 
 Argument        : void
*******************************************************************************/
i32 EWL_WaitHwRdy(void)
{

#ifdef HMP4_NO_TIMEOUT
    int sig, err;

    err = sigwait(&enc->sigset, &sig);
    if(err || sig != SIGIO)
    {
        PTRACE("EWL_WaitHwRdy: Unknown signal received! %i\n", sig);
        return EWL_NOK;
    }
    return (EWL_OK);
#else
    int sig;
    struct timespec to;
    siginfo_t si;

    to.tv_sec = 2;
    to.tv_nsec = 0;

    sig = sigtimedwait(&enc->sigset, &si, &to);

    if(sig != SIGIO)
    {
        if(errno == EAGAIN)
        {
            PTRACE("EWL_WaitHwRdy: Hardware timeout! %i\n", sig);
            return EWL_TIMEOUT;
        }
        else
        {
            PTRACE("EWL_WaitHwRdy:Unknown signal received! %i\n", sig);
            return EWL_NOK;
        }
    }

    return (EWL_OK);
#endif
}

/*******************************************************************************
 Function name   : EWL_ResetHwRdy
 Description     : Reset the enocoder semaphore to the unsignaled state
 Return type     : ewl_ret 
 Argument        : void
*******************************************************************************/
i32 EWL_ResetHwRdy(void)
{
    sigset_t ss;

    sigpending(&ss);

    if(sigismember(&ss, SIGIO))
    {
        int sig;

        if(sigwait(&enc->sigset, &sig) != 0) ;
        {
            PTRACE("EWL_ResetHwRdy: Unknown signal received! %i\n", sig);
            return EWL_NOK;
        }
    }
    return (EWL_OK);
}
